#ifndef _FOURIERFARM_H_
#define _FOURIERFARM_H_

#include <complex>
using namespace std;

class FourierFarm;
class FourierFarm2D;

// Pure Filter Kernel class
class FourierKernel
	{
	protected:
		FourierFarm *m_fFarm;

		FourierKernel(int np) {m_np = np;m_pkernelf = NULL;m_pkernelc = NULL;}
	public:
		int m_np;
		float *m_pkernelf;
		complex<float> *m_pkernelc;

		virtual ~FourierKernel() {}

		int getNumPoints() {return m_np;}

		virtual inline void setImpuls(float *filtimp) = 0;
		virtual inline void setTransf(complex<float> *filttra) = 0;
	};

// Pure Fourier Farm class
class FourierFarm
	{
	protected:
		int m_np;
		bool m_isPad;

	public:
		// Pi value as used by a Fourier Farm
		const float m_pi;
		// Query the number of points needed to fourier transform a series of
		// npToPad points with/without zero padding
		virtual int queryPad(int npToPad,bool isPad = true) = 0;
		// Alloc a new filter kernel
		virtual FourierKernel* allocKernel() = 0;

		FourierFarm(int npToPad,bool isPad) : m_pi(3.14159265359F) {m_np = 0; m_isPad = false;};
		virtual ~FourierFarm() {};

		int getNumPoints(void) {return m_np;}
		bool getPad(void) {return m_isPad;}

		// 1D FFT and IFFT with input buffer rewritten for output
		virtual inline int fft_1D(float *inout)= 0;
		virtual inline int ifft_1D(float *inout) = 0;
		// 1D FFT and IFFT with output buffers
		virtual inline int fft_1D(float *in,float *out) = 0;
		virtual inline int ifft_1D(float *in,float *out) = 0;

		virtual inline int fft_1D(complex<float> *inout)= 0;
		virtual inline int ifft_1D(complex<float> *inout) = 0;
		virtual inline int fft_1D(complex<float> *in,complex<float> *out) = 0;
		virtual inline int ifft_1D(complex<float> *in,complex<float> *out) = 0;

		virtual inline int fft_1Ds(float *inout_re,float *inout_im) = 0;
		virtual inline int ifft_1Ds(float *inout_re,float *inout_im) = 0;

		virtual inline filter_1D(float *inout,FourierKernel*kern) = 0;
		virtual inline filter_1D(complex<float> *inout,FourierKernel*kern) = 0;
		virtual inline filter_1Ds(float *inout_re,float *inout_im,FourierKernel*kern) = 0;
	};

// Pure Filter Kernel 2D class
class FourierKernel2D
	{
	protected:
		FourierFarm2D *m_fFarm;

		FourierKernel2D(int npx,int npy) {m_npx = npx;m_npy = npy;m_pkernelf = NULL;m_pkernelc = NULL;}
	public:
		int m_npx,m_npy;
		float *m_pkernelf;
		complex<float> *m_pkernelc;

		virtual ~FourierKernel2D() {}

		int getNumPoints() {return m_npx*m_npy;}

		virtual inline void setImpuls(float *filtimp) = 0;
		virtual inline void setTransf(complex<float> *filttra) = 0;
	};

// Pure Fourier Farm 2D class
class FourierFarm2D
	{
	protected:
		int m_npx,m_npy;
		bool m_isPad;

	public:
		// Pi value as used by a Fourier Farm 2D
		const float m_pi;
		// Query the number of points needed to fourier transform a series of
		// npxToPad x npyToPad points with/without zero padding
		virtual void queryPad(int npxToPad,int npyToPad,int &npx,int &npy,bool isPad = true) = 0;
		// Alloc a new filter kernel
		virtual FourierKernel2D* allocKernel() = 0;

		FourierFarm2D(int npxToPad,int npyToPad,bool isPad) : m_pi(3.14159265359F) {m_npx = m_npy = 0; m_isPad = false;}
		virtual ~FourierFarm2D() {}

		int getNumXPoints(void) {return m_npx;}
		int getNumYPoints(void) {return m_npy;}
		bool getPad(void) {return m_isPad;}

		// 2D FFT and IFFT with input buffer rewritten for output
		virtual inline int fft_2D(float *inout)= 0;
		virtual inline int ifft_2D(float *inout) = 0;
		// 2D FFT and IFFT with output buffers
		virtual inline int fft_2D(float *in,float *out) = 0;
		virtual inline int ifft_2D(float *in,float *out) = 0;
/*
		virtual inline int fft_1D(complex<float> *inout)= 0;
		virtual inline int ifft_1D(complex<float> *inout) = 0;
		virtual inline int fft_1D(complex<float> *in,complex<float> *out) = 0;
		virtual inline int ifft_1D(complex<float> *in,complex<float> *out) = 0;

		virtual inline int fft_1Ds(float *inout_re,float *inout_im) = 0;
		virtual inline int ifft_1Ds(float *inout_re,float *inout_im) = 0;
*/
		virtual inline filter_2D(float *inout,FourierKernel2D *kern) = 0;
//		virtual inline filter_1D(complex<float> *inout,FourierKernel*kern) = 0;
//		virtual inline filter_1Ds(float *inout_re,float *inout_im,FourierKernel*kern) = 0;

	};

#ifndef FOURIERFARM_EXPORTS
 __declspec(dllimport) FourierFarm *GetFourierFarm_Best(int npToPad,bool isPad);
 __declspec(dllimport) FourierFarm2D *GetFourierFarm2D_Best(int npxToPad,int npyToPad,bool isPad);
#endif

#endif